"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_line_args_1 = __importDefault(require("command-line-args"));
const express_1 = __importDefault(require("express"));
const typescript_lambda_api_1 = require("typescript-lambda-api");
/**
 * Simple console application that hosts an express HTTP
 * server.
 *
 * A request is mapped from the HTTP request to the ApiServer
 * event format, a ApiServer instance then processes the request.
 * The ApiServer response is mapped to a standard HTTP response and
 * returned to the express client.
 */
class ApiConsoleApp extends typescript_lambda_api_1.ApiApp {
    constructor(controllersPath, appConfig, appContainer) {
        super(appConfig, appContainer);
        this.controllersPath = controllersPath;
        this.expressApp = express_1.default();
    }
    /**
     * Starts the express server.
     *
     * @param args Command line arguments for this server, see --help for more info.
     */
    async run(args) {
        let self = this;
        let options = this.parseArguments(args);
        await super.initialiseControllers(this.controllersPath);
        this.expressApp.all("*", (req, res, next) => self.handleHttpRequest(self, req, res, next));
        this.expressApp.listen(options.port, options.host, () => {
            console.log(`App is now listening for HTTP requests on http://${options.host}:${options.port} ...`);
        });
    }
    parseArguments(args) {
        return command_line_args_1.default(ApiConsoleApp.APP_OPTIONS, {
            argv: args
        });
    }
    async handleHttpRequest(self, request, response, onError) {
        try {
            let apiRequestEvent = await self.mapRequestToApiEvent(request);
            let apiResponse = await self.apiServer.processEvent(apiRequestEvent, {});
            self.forwardApiResponse(apiResponse, response);
        }
        catch (e) {
            onError(e);
        }
    }
    async mapRequestToApiEvent(request) {
        let apiRequest = new typescript_lambda_api_1.ApiRequest();
        apiRequest.httpMethod = request.method;
        apiRequest.path = request.path;
        Object.keys(request.headers)
            .forEach(h => apiRequest.headers[h] = request.headers[h]);
        Object.keys(request.query)
            .forEach(q => apiRequest.queryStringParameters[q] = request.query[q]);
        // read request body (if any) as a base 64 string
        return await new Promise((resolve, reject) => {
            let body = "";
            request.setEncoding("base64");
            request.on("data", d => body += d);
            request.on("end", () => {
                apiRequest.body = body;
                apiRequest.isBase64Encoded = true;
                resolve(apiRequest);
            });
            request.on("error", reject);
        });
    }
    forwardApiResponse(apiResponse, response) {
        let headers = apiResponse.headers;
        response.status(apiResponse.statusCode);
        Object.keys(headers).forEach(h => response.header(h, headers[h]));
        let body = apiResponse.body;
        if (apiResponse.isBase64Encoded) {
            body = Buffer.from(body, 'base64');
        }
        response.send(body);
    }
}
ApiConsoleApp.APP_OPTIONS = [
    { name: 'port', alias: 'p', type: Number, defaultValue: 5555 },
    { name: 'host', alias: 'h', type: String, defaultValue: "127.0.0.1" }
];
__decorate([
    typescript_lambda_api_1.timed,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [ApiConsoleApp, Object, Object, Function]),
    __metadata("design:returntype", Promise)
], ApiConsoleApp.prototype, "handleHttpRequest", null);
exports.ApiConsoleApp = ApiConsoleApp;
//# sourceMappingURL=ApiConsoleApp.js.map